#! /usr/bin/env python

import base64
import binascii
import crypt
import logging
import hashlib
import json
import os
import time
import tempfile
import urllib.request
import urllib.error
import subprocess
import sys

TIME_OUT = 5
RETRY_NUM = 60

METADATA_URL = 'http://169.254.169.254/openstack/latest/meta_data.json'
USERDATA_URL = 'http://169.254.169.254/openstack/latest/user_data'
RESETPWD_FLAG_URL = 'http://169.254.169.254/openstack/latest/resetpwd_flag'
RESET_PASSWORD_URL = 'http://169.254.169.254/openstack/latest/reset_password'
RESETPWD_FLAG = 'resetpwd_flag'
RESET_PASSWORD = 'reset_password'
PASSWD_POSITION_FLAG_IN_USERDATA = 1
REQUEST_INTERVAL_TIMES = 2

VPC_MAC_NUM = 2

ROOT_PASSWD_FILE = '/etc/shadow'
AUTHORIZED_KEYS_PATH = "/etc/ssh/keys-root/"
AUTHORIZED_KEYS_FILE = "/etc/ssh/keys-root/authorized_keys"
SSH_CONFIG_FILE = "/etc/ssh/sshd_config"

LOG = logging.getLogger()
handler = logging.FileHandler("/var/log/network-config.log")
formatter = logging.Formatter('%(asctime)s - '
                              '%(filename)s[%(levelname)s]: %(message)s')
handler.setFormatter(formatter)
LOG.addHandler(handler)
LOG.setLevel(logging.INFO)


def get_configdirve_local_path():
    find_local_configdrive_cmd = "find / -name configdrive"
    (status, output) = subprocess.getstatusoutput(find_local_configdrive_cmd)
    if status != 0:
        LOG.error("get local configdrive failed, reason:%s" % output)
        return False, None
    if not output:
        LOG.error("configdrive is not exist")
        return False, None
    LOG.info("configdrive path is: %s", output)
    return True, output


def get_network_data_from_configgrive():
    ret, configdrive_path = get_configdirve_local_path()
    if not ret:
        LOG.error("get configdrive path failed.")
        return False
    network_data_file = configdrive_path + "/openstack/latest/network_data.json"
    try:
        network_data = open(network_data_file)
        network_json = json.load(network_data)
        if not network_json:
            LOG.error("get esxi network json failed.")
            return False, None
        return True, network_json
    except Exception as e:
        LOG.error("open %s. err: %s" % (network_data_file, e))
        return False, network_json
    finally:
        network_data.close()


def get_mac_from_configdirve(network_data, linktype):
    if not network_data:
        LOG.error("get network data file failed.")
        return False, None
    links_info = network_data.get('links', None)
    if not links_info:
        LOG.error("get links info failed.")
        return False, None
    mac_total = []
    for link in links_info:
        if link.get('type', None) == linktype and  \
                link.get('ethernet_mac_address', None):
            mac_total.append(link.get('ethernet_mac_address'))
    if not mac_total:
        LOG.error("get mac info failed.")
        return False, None
    return True, mac_total


def get_phy_mac_from_configdrive(network_data):
    if not network_data:
        LOG.error("get network data file failed.")
        return False, None

    ret, ifphy_mac_total = get_mac_from_configdirve(network_data, 'phy')
    if not ret:
        LOG.error("get phy mac info failed.")
        return False, None

    if len(ifphy_mac_total) != VPC_MAC_NUM:
        LOG.error("esxi phy nic num is not correct.")
        return False

    return True, ifphy_mac_total


def get_vpc_mac_from_configdrive(network_data):
    if not network_data:
        LOG.error("get network data file failed.")
        return False, None
    ret, vpc_mac = get_mac_from_configdirve(network_data, 'bond')
    if not ret:
        LOG.error("get phy mac info failed.")
        return False, None
    return True, vpc_mac


def get_physical_nic_name(phy_mac):
    if not phy_mac:
        LOG.error("esxi phy nic mac not exist.")
        return False, None
    get_ifphy_name_cmd = "esxcli network nic list | grep %s | awk '{print $1}' " % phy_mac
    (status, output) = subprocess.getstatusoutput(get_ifphy_name_cmd)
    if status != 0:
        LOG.error("get esxi phy nic name failed, reason:%s" % output)
        return False, None
    if not output:
        LOG.error("esxi phy nic name not exist.")
        return False, None
    return True, output


def remove_nic_form_vswitch():
    get_all_phy_nic_cmd = "esxcli network nic list | grep vmnic | awk '{print $1}'"
    (status, output) = subprocess.getstatusoutput(get_all_phy_nic_cmd)
    if status != 0:
        LOG.error("get all esxi phy nic failed, reason:%s" % output)
        return False
    if not output:
        LOG.error("esxi phy nic not exist.")
        return False
    phy_nics = output.split('\n')
    for phy_nic in phy_nics:
        remove_nic_from_vswitch0_cmd = "esxcli network vswitch standard uplink remove -u %s -v vSwitch0" % phy_nic
        (status, output) = subprocess.getstatusoutput(remove_nic_from_vswitch0_cmd)
        LOG.info("remove nic: %s form vswitch0 , status: %s, output: %s" % (phy_nic, status, output))
    return True


def add_vpc_nic_to_vswitch(phy_nic_name):
    if not phy_nic_name:
        LOG.error("vpc phy nic name not exist.")
        return False
    add_vpc_nic_to_vswitch_cmd = "esxcli network vswitch standard uplink add -u %s -v vSwitch0" % phy_nic_name
    (status, output) = subprocess.getstatusoutput(add_vpc_nic_to_vswitch_cmd)
    if status != 0:
        LOG.error("add vpc phy nic to vswicth failed, reason:%s" % output)
        return False
    return True


def update_vpc_mac_to_vmk0(bond_mac):
    if not bond_mac:
        LOG.error("bond mac not exist.")
        return False

    clear_ip_info_from_vmk0_cmd = "esxcli network ip interface remove -i vmk0"
    (status, output) = subprocess.getstatusoutput(clear_ip_info_from_vmk0_cmd)
    if status != 0:
        LOG.error("clear vmk0 ip info failed, reason:%s" % output)

    update_vmk0_mac_cmd = "esxcli network ip interface add -i vmk0 -M %s -p 'Management Network'" % bond_mac[0]
    (status, output) = subprocess.getstatusoutput(update_vmk0_mac_cmd)
    if status != 0:
        LOG.error("update vmk0 mac info failed, reason:%s" % output)
        return False

    set_vmk0_dhcp_mode_cmd = "esxcli network ip interface ipv4 set -i vmk0 -t dhcp"
    (status, output) = subprocess.getstatusoutput(set_vmk0_dhcp_mode_cmd)
    if status != 0:
        LOG.error("set vmk0 dhcp failed, reason:%s" % output)
        return False
    return True


def create_first_boot_flag():
    ret, configdrive_path = get_configdirve_local_path()
    if not ret:
        LOG.error("get configdrive path failed.")
        return False
    try:
        first_boot_flag_file = open(configdrive_path + "/first_boot_flag", "w+")
        first_boot_flag_file.write("True")
        return True
    except Exception as e:
        LOG.error("create first boot flag. err: %s" % e)
        return False
    finally:
        first_boot_flag_file.close()


def esxi_network_config():
    ret = create_first_boot_flag()
    if not ret:
        LOG.error("create first boot flag failed.")

    ret, network_json = get_network_data_from_configgrive()
    if not ret:
        LOG.error("get network json data file failed.")
        return False

    ret, ifphy_mac_total = get_phy_mac_from_configdrive(network_json)
    if not ret:
        LOG.error("get phy nic mac info failed.")
        return False

    ret, vpc_mac = get_vpc_mac_from_configdrive(network_json)
    if not ret:
        LOG.error("get vpc mac info failed.")
        return False

    ret, first_phy_name = get_physical_nic_name(ifphy_mac_total[0])
    if not ret:
        LOG.error("get first phy nic name failed.")
        return False

    ret, second_phy_name = get_physical_nic_name(ifphy_mac_total[1])
    if not ret:
        LOG.error("get first phy nic name failed.")
        return False

    ret = remove_nic_form_vswitch()
    if not ret:
        LOG.error("remove nic from vswitch failed.")
        return False

    ret = add_vpc_nic_to_vswitch(first_phy_name)
    if not ret:
        LOG.error("add first vpc phy nic to vswitch failed.")
        return False

    ret = add_vpc_nic_to_vswitch(second_phy_name)
    if not ret:
        LOG.error("add second vpc phy nic to vswitch failed.")
        return False

    ret = update_vpc_mac_to_vmk0(vpc_mac)
    if not ret:
        LOG.error("update vpc mac to vmk0 failed.")
        return False

    return True


def readurl(url, outputfile=None, retries=0, sec_between=1):
    manual_tries = 1
    if retries:
        manual_tries = max(int(retries) + 1, 1)

    if sec_between is None:
        sec_between = -1

    for i in range(0, manual_tries):
        try:
            LOG.debug("[%s/%s] open '%s' ", i,
                      manual_tries, url)
            cmd = "wget %s -O %s" % (url, outputfile)
            (status, output) = subprocess.getstatusoutput(cmd)
            LOG.info("try to get %s,status is %s,output is %s" % (url, status, output))
            if status != 0:
                LOG.error("get %s failed, reason:%s" % (url, output))
                if i + 1 < manual_tries and sec_between > 0:
                    LOG.debug("Please wait %s seconds while we wait to try again", sec_between)
                    time.sleep(sec_between)
                else:
                    LOG.error("try to get %s Timeout." % url)
                    return False
            else:
                return True
        except Exception as e:
            LOG.error("get %s failed, Error:%s", url, e)
            return False

    return True


def get_esxi_data(tmpfile, retries=RETRY_NUM, url=METADATA_URL):
    try:
        ret = readurl(url=url,
                      outputfile=tmpfile,
                      retries=retries,
                      sec_between=REQUEST_INTERVAL_TIMES
                      )

        if not ret:
            LOG.error("download data failed.")
            return False
    except Exception as e:
        LOG.error("Get data from metadata server %s failed. Error: %s", METADATA_URL, e)
        return False
    return True


def get_esxi_metadata():
    try:
        metadata_tmpfile = tempfile.NamedTemporaryFile()
        ret = get_esxi_data(metadata_tmpfile.name, RETRY_NUM, METADATA_URL)
        if not ret:
            LOG.error("download metadata failed.")
            return False, None
        else:
            metadata = json.load(open(metadata_tmpfile.name))
            return True, metadata
    finally:
        metadata_tmpfile.close()


def get_esxi_userdata():
    try:
        userdata_tmpfile = tempfile.NamedTemporaryFile()
        ret = get_esxi_data(userdata_tmpfile.name, RETRY_NUM, USERDATA_URL)
        with open(userdata_tmpfile.name, "r+") as userdata_file:
            userdata = userdata_file.readlines()
        if not ret:
            LOG.error("download userdata failed.")
            return False, None          
        else:
            return True, userdata           
    except Exception as e:
        LOG.error("Get userdata from metadata server %s failed. Error: %s", USERDATA_URL, e)
        return False, None
    finally:
        userdata_file.close()
        userdata_tmpfile.close()


def get_esxi_hostname(metadata):
    if not metadata:
        LOG.error("get metadata failed.")
        return False, None
    else:
        hostname = metadata.get('name', None)
    if not hostname:
        LOG.error("Get hostname' info failed")
        return False, None
    return True, hostname


def get_esxi_authorized_keys(metadata):
    if not metadata:
        LOG.error("get metadata failed.")
        return False, None
    else:
        if metadata.get('public_keys', None):
            authorized_keys = list(metadata.get('public_keys', None).values())[0]
        else:
            authorized_keys = None
    if not authorized_keys:
        LOG.info("Get authorized_keys' info failed")
        return False, None
    return True, authorized_keys.strip('\n')


def get_esxi_passwd():
    ret, userdata = get_esxi_userdata()
    if not ret:
        LOG.error("get userdata failed.")
        return False, None
    else:
        userdata_length = len(userdata)
        if (userdata_length == 2) and ('chpasswd' in userdata[PASSWD_POSITION_FLAG_IN_USERDATA]):
            passwd = userdata[PASSWD_POSITION_FLAG_IN_USERDATA].split("'")[PASSWD_POSITION_FLAG_IN_USERDATA]\
                     .split(":")[PASSWD_POSITION_FLAG_IN_USERDATA]
            return True, passwd
        else:
            LOG.error("passwd info not in the user data.")
            return False, None


def set_esxi_hostname(hostname):
    if not hostname:
        LOG.error("set esxi hostname failed, hostname not exist.")
        return False
    cmd = "esxcli system hostname set --host" + " " + hostname
    (status, output) = subprocess.getstatusoutput(cmd)
    LOG.info("try to set esxi hostname %s, status is %s,output is %s" % (hostname, status, output))
    if status != 0:
        LOG.error("set esxi hostname failed, output is %s" % output)
        return False
    return True


def replace_str_in_file(file, old_str, new_str):
    file_data = ""
    try:
        with open(file, "r") as f:
            for line in f:
                if old_str in line:
                    line = line.replace(old_str, new_str)
                file_data += line
        with open(file, "w") as f:
            f.write(file_data)
        return True
    except Exception as e:
        LOG.error("open %s failed. err: %s" % (file, e))
        return False
    finally:
        f.close()


def set_esxi_passwd_by_shadow_file(passwd):
    if not passwd:
        LOG.error("set esxi paasswd failed, passwd not exist.")
        return False
    try:
        shadow_file = open(ROOT_PASSWD_FILE, mode='r')
        if shadow_file:
            rootinfo = shadow_file.readlines()[0]
            if len(rootinfo) > 2:
                origin_passwd = rootinfo.split(":")[1]
            else:
                LOG.error("get root passwd info from /etc/shadow failed.")
        else:
            LOG.error("read /etc/shadow failed.")
        if origin_passwd:
            ret = replace_str_in_file(ROOT_PASSWD_FILE, origin_passwd, passwd)
            if not ret:
                LOG.error("update /etc/shadow failed.")
                return False
    except Exception as e:
        LOG.error("open /etc/shadow failed. err: %s" % e)
        return False
    finally:
        shadow_file.close()
    return True


def set_esxi_authorized_keys(authorized_keys):
    if not authorized_keys:
        LOG.info("set esxi authorized keys failed, keys not exist.")
        return False
    if not os.path.exists(AUTHORIZED_KEYS_PATH):
        os.makedirs(AUTHORIZED_KEYS_PATH)
    if os.path.exists(AUTHORIZED_KEYS_FILE):
        os.remove(AUTHORIZED_KEYS_FILE)
    cmd = "echo " + authorized_keys + "> " + AUTHORIZED_KEYS_FILE
    (status, output) = subprocess.getstatusoutput(cmd)
    LOG.info("write authorized keys in esxi, status is %s,output is %s" % (status, output))
    if status != 0:
        LOG.error("write authorized keys in esxi failed, output is %s" % output)
        return False
    return True


def config_ssh_service():
    cmd_permit_root_login = "sed -i 's/^PermitRootLogin.*/PermitRootLogin yes/' %s" % SSH_CONFIG_FILE
    (status, output) = subprocess.getstatusoutput(cmd_permit_root_login)
    LOG.info("set permit root login in esxi, status is %s,output is %s" % (status, output))
    if status != 0:
        LOG.error("set permit root login in esxi failed, output is %s" % output)
        return False

    cmd_permit_passwd_login = "sed -i 's/^PasswordAuthentication.*/PasswordAuthentication yes/' %s" % SSH_CONFIG_FILE
    (status, output) = subprocess.getstatusoutput(cmd_permit_passwd_login)
    LOG.info("set permit passwd auth in esxi, status is %s,output is %s" % (status, output))
    if status != 0:
        LOG.error("set permit passwd auth in esxi failed, output is %s" % output)
        return False

    cmd_permit_response_auth = "sed -i 's/^ChallengeResponseAuthentication.*/ChallengeResponseAuthentication no/' %s" \
                               % SSH_CONFIG_FILE
    (status, output) = subprocess.getstatusoutput(cmd_permit_response_auth)
    LOG.info("set permit response auth in esxi, status is %s,output is %s" % (status, output))
    if status != 0:
        LOG.error("set permit response auth in esxi failed, output is %s" % output)
        return False

    cmd_restart_ssh_service = "/etc/init.d/SSH restart"
    (status, output) = subprocess.getstatusoutput(cmd_restart_ssh_service)
    LOG.info("restart ssh service in esxi, status is %s,output is %s" % (status, output))
    if status != 0:
        LOG.error("restart ssh service in esxi failed, output is %s" % output)
        return False

    return True


def persist_configuration_in_esxi():
    cmd = "/bin/sh /sbin/auto-backup.sh"
    (status, output) = subprocess.getstatusoutput(cmd)
    LOG.info("try to persist configuration in esxi, status is %s,output is %s" % (status, output))
    if status != 0:
        LOG.error("persist configuration in esxi failed, output is %s" % output)
        return False
    return True


def get_esxi_host_uuid(metadata):
    if not metadata:
        LOG.error("get metadata failed.")
        return False, None
    else:
        host_uuid = metadata.get('uuid', None)
    if not host_uuid:
        LOG.info("Get host_uuid failed")
        return False, None
    return True, host_uuid


def process_esxi_metadata_info():
    ret, metadata_json = get_esxi_metadata()
    if not ret:
        LOG.error("get metadata failed.")
        return False

    ret, hostname = get_esxi_hostname(metadata_json)
    if not ret:
        LOG.error("get hostname failed.")

    ret, esxi_authorized_keys = get_esxi_authorized_keys(metadata_json)
    if not ret:
        LOG.error("get authorized keys failed.")

    ret, passwd = get_esxi_passwd()
    if not ret:
        LOG.error("get esxi passwd failed.")

    ret = set_esxi_hostname(hostname)
    if not ret:
        LOG.error("set esxi hostname failed.")

    ret = set_esxi_authorized_keys(esxi_authorized_keys)
    if not ret:
        LOG.error("set authorized keys failed.")

    ret = set_esxi_passwd_by_shadow_file(passwd)
    if not ret:
        LOG.error("set esxi passwd failed.")

    ret = config_ssh_service()
    if not ret:
        LOG.error("config ssh service failed.")

    ret = persist_configuration_in_esxi()
    if not ret:
        LOG.error("persist esxi conf failed.")

    return True


def get_reset_passwd_flag_alone():
    try:
        reset_passwd_flag_resp = urllib.request.urlopen(RESETPWD_FLAG_URL, data=None, timeout=TIME_OUT)
        reset_passwd_flag_json = json.loads(reset_passwd_flag_resp.read().decode('utf8'))

        if not reset_passwd_flag_json:
            return None
        else:
            reset_passwd_flag = reset_passwd_flag_json.get(RESETPWD_FLAG, None)
            return reset_passwd_flag
    except urllib.error.URLError as e:
        if hasattr(e, 'reason'):
            LOG.error('URL Error reason: %s' % e.reason)
        return None
    except urllib.error.HTTPError as e:
        if hasattr(e, 'code'):
            LOG.error('http err Reason:%s' % e.code)
        return None
    finally:
        if reset_passwd_flag_resp:
            reset_passwd_flag_resp.close()


def get_reset_passwd_flag():
    for i in range(0, RETRY_NUM):
        reset_passwd_flag = get_reset_passwd_flag_alone()
        if reset_passwd_flag and reset_passwd_flag != "None":
            return True, reset_passwd_flag
        else:
            if i + 1 < RETRY_NUM:
                LOG.info("Please wait %s seconds while we wait to try again", REQUEST_INTERVAL_TIMES)
                time.sleep(REQUEST_INTERVAL_TIMES)
            else:
                LOG.error("try to get get_reset_passwd_flag Timeout.")
                return False, None


def set_reset_passwd_flag():
    try:
        for i in range(0, RETRY_NUM):
            agent_installed_flag = "True"
            flag_to_byte = urllib.parse.unquote_to_bytes(agent_installed_flag)
            urllib.request.urlopen(RESETPWD_FLAG_URL, data=flag_to_byte, timeout=TIME_OUT)
            reset_passwd_flag = get_reset_passwd_flag_alone()
            if str(reset_passwd_flag) == "True":
                return True
            else:
                if i + 1 < RETRY_NUM:
                    LOG.info("Please wait %s seconds while we wait to try again", REQUEST_INTERVAL_TIMES)
                    time.sleep(REQUEST_INTERVAL_TIMES)
                else:
                    LOG.error("try to set passwd_flag Timeout.")
                    return False
    except Exception as e:
        LOG.error("set passwd flag err:%s" % e)
        return False


def get_reset_passwd_alone():
    try:
        reset_passwd_resp = urllib.request.urlopen(RESET_PASSWORD_URL, data=None, timeout=TIME_OUT)
        reset_passwd_json = json.loads(reset_passwd_resp.read().decode('utf8'))
        if not reset_passwd_json:
            return None
        else:
            reset_passwd = reset_passwd_json.get(RESET_PASSWORD, None)
            return reset_passwd
    except urllib.error.URLError as e:
        if hasattr(e, 'reason'):
            LOG.error('URL Error reason: %s' % e.reason)
        return None
    except urllib.error.HTTPError as e:
        if hasattr(e, 'code'):
            LOG.error('http err Reason:%s' % e.code)
        return None
    finally:
        if reset_passwd_resp:
            reset_passwd_resp.close()


def get_reset_passwd():
    for i in range(0, RETRY_NUM):
        reset_passwd = get_reset_passwd_alone()
        if reset_passwd and len(reset_passwd) > 12:
            return True, reset_passwd
        else:
            if i + 1 < RETRY_NUM:
                LOG.info("Please wait %s seconds while we wait to try again", REQUEST_INTERVAL_TIMES)
                time.sleep(REQUEST_INTERVAL_TIMES)
            else:
                LOG.error("try to get get_reset_passwd_flag Timeout.")
                return False, None


def get_sault(encrypted_data):
    if not encrypted_data or (len(encrypted_data) < 12):
        return None
    return encrypted_data[:12]


def get_aes_key(host_uuid, encrypted_data):
    if not host_uuid:
        LOG.error("host uuid not exist.")
        return False, None

    iterations = 5000
    salt = get_sault(encrypted_data)

    if not salt:
        LOG.error("salt value not exist.")
        return False, None

    salt_base64_decrypt = base64.b64decode(salt)
    aes_key = hashlib.pbkdf2_hmac("sha1", bytes(host_uuid, encoding="utf8"), salt_base64_decrypt, iterations, 16)
    if not aes_key:
        LOG.error("aes key not exist.")
        return False, None
    return True, aes_key


def write_encrypt_reset_password(encrypted_data, encrypted_data_tmpfile):
    try:
        encrypted_data_base64_decrypt = base64.b64decode(encrypted_data)
        encrypted_data_file = open(encrypted_data_tmpfile, 'wb+')
        encrypted_data_file.write(encrypted_data_base64_decrypt)
    except Exception as e:
        LOG.error("Write encrypted pwd data_file conf error:%s" % e)
        return False
    finally:
        encrypted_data_file.close()
        LOG.info("Write file %s Done" % encrypted_data_tmpfile)
        return True


def aes_cbc_decrypt(encrypted_data_tmpfile, decrypted_data_tmpfile, aes_key):
    try:
        key_hex = binascii.hexlify(aes_key).decode()
        passwd_decrypt_cmd = "openssl aes-128-cbc -d -in %s -out %s -iv %s -K %s" % \
                             (encrypted_data_tmpfile, decrypted_data_tmpfile, key_hex, key_hex)
        (status, output) = subprocess.getstatusoutput(passwd_decrypt_cmd)
        if status != 0:
            LOG.error("passwd decrypt failed, output is %s" % output)
            return False
    except Exception as e:
        LOG.error("passwd decrypt failed, error:%s" % e)
        return False
    return True


def reset_esxi_passwd_by_shadow(decrypted_data_tmpfile):
    try:
        passwd = open(decrypted_data_tmpfile, mode='r')
        if passwd:
            passwdinfo = passwd.readlines()[0]
        if not passwdinfo:
            LOG.error("get passwdinfo failed, output is ")
            return False

        encry_passwd = crypt.crypt(passwdinfo)
        ret = set_esxi_passwd_by_shadow_file(encry_passwd)
        if not ret:
            LOG.error("reset esxi passwd failed.")
    except Exception as e:
        LOG.error("open %s failed, err %s" % (decrypted_data_tmpfile, e))
    finally:
        passwd.close()


def delete_esxi_metadata_passwd():
    try:
        for i in range(0, RETRY_NUM):
            reset_passwd_resp = urllib.request.Request(RESET_PASSWORD_URL)
            reset_passwd_resp.get_method = lambda: 'DELETE'
            urllib.request.urlopen(reset_passwd_resp)
            esxi_metadata_passwd = get_reset_passwd_alone()
            if not esxi_metadata_passwd:
                LOG.info("clear passwd from metadata server success.")
                return True
            if i + 1 < RETRY_NUM:
                LOG.info("Please wait %s seconds while we wait to try again", REQUEST_INTERVAL_TIMES)
                time.sleep(REQUEST_INTERVAL_TIMES)
            else:
                LOG.error("del passwd from metadata server timeout.")
                return False
    except Exception as e:
        LOG.error("delete esxi passwd from metadata server err:%s" % e)
        return False


def reset_esxi_passwd():
    try:
        encrypted_data_tmpfile = tempfile.NamedTemporaryFile()
        decrypted_data_tmpfile = tempfile.NamedTemporaryFile()

        ret, reset_passwd_flag = get_reset_passwd_flag()
        if not ret:
            LOG.error("get reset_passwd_flag failed.")

        if not reset_passwd_flag:
            ret = set_reset_passwd_flag()
            if not ret:
                LOG.error("set reset_passwd_flag failed.")

        ret, passwd = get_reset_passwd()
        if not ret:
            LOG.error("get reset passwd failed.")
            return False

        ret, metadata_json = get_esxi_metadata()
        if not ret:
            LOG.error("get metadata failed.")
            return False

        ret, host_uuid = get_esxi_host_uuid(metadata_json)
        if not ret:
            LOG.error("get host uuid failed.")
            return False

        ret, aes_key = get_aes_key(host_uuid, passwd)
        if not ret:
            LOG.error("get aes key failed.")
            return False

        decrypted_password = passwd[12:]

        ret = write_encrypt_reset_password(decrypted_password, encrypted_data_tmpfile.name)
        if not ret:
            LOG.error("write encrypt reset password failed.")
            return False

        ret = aes_cbc_decrypt(encrypted_data_tmpfile.name, decrypted_data_tmpfile.name, aes_key)
        if not ret:
            LOG.error("decrypt esxi password failed.")
            return False

        ret = reset_esxi_passwd_by_shadow(decrypted_data_tmpfile.name)
        if not ret:
            LOG.error("reset esxi passwd failed.")
            return False

        ret = persist_configuration_in_esxi()
        if not ret:
            LOG.error("persist esxi conf failed.")

        ret = delete_esxi_metadata_passwd()
        if not ret:
            LOG.error("del esxi password from metadata failed.")

        return True

    except Exception as e:
        LOG.error("decrypt password failed. err: %s" % e)
    finally:
        encrypted_data_tmpfile.close()
        decrypted_data_tmpfile.close()


def is_first_boot_flag_exist():
    ret, configdrive_path = get_configdirve_local_path()
    if not ret:
        LOG.error("get configdrive path failed.")

    first_boot_flag_file = configdrive_path + "/first_boot_flag"
    return not os.path.isfile(first_boot_flag_file)


def main():
    try:
        is_first_boot_ret = is_first_boot_flag_exist()
        if is_first_boot_ret:
            ret = esxi_network_config()
            if not ret:
                LOG.error("config esxi network failed.")

            ret = process_esxi_metadata_info()
            if not ret:
                LOG.error("config esxi metadata failed.")

        ret = reset_esxi_passwd()
        if not ret:
            LOG.error("reset esxi passwd failed.")
    except Exception as e:
        LOG.error("configure esxi failed:%s" % e)

    LOG.info("configure esxi end.")


if __name__ == "__main__":
    sys.exit(main())
